﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Dapfor.Wpf.Controls;

namespace Dapfor.Wpf.Grid.Demo.Examples.DataBinding.ConditionalHierarchicalBinding
{
    public class StatusCustomControl : Button
    {
        #region Cell dependency property

        internal static readonly DependencyProperty CellProperty = DependencyProperty.Register("Cell", typeof (Cell), typeof (StatusCustomControl), new PropertyMetadata(OnCellChanged));

        internal Cell Cell
        {
            get { return (Cell) GetValue(CellProperty); }
            set { SetValue(CellProperty, value); }
        }

        private static void OnCellChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (StatusCustomControl) d;
            control.UpdateStatus();
        }

        #endregion

        #region Value dependency property

        internal static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", typeof (object), typeof (StatusCustomControl), new PropertyMetadata(OnValueChanged));

        internal object Value
        {
            get { return GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        private static void OnValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (StatusCustomControl) d;
            control.UpdateStatus();
        }

        #endregion

        #region ActiveColor dependency property

        internal static readonly DependencyProperty ActiveColorProperty = DependencyProperty.Register("ActiveColor", typeof (Color), typeof (StatusCustomControl), new PropertyMetadata(OnActiveColorChanged));

        internal Color ActiveColor
        {
            get { return (Color) GetValue(ActiveColorProperty); }
            set { SetValue(ActiveColorProperty, value); }
        }

        private static void OnActiveColorChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (StatusCustomControl) d;
            control.UpdateStatus();
        }

        #endregion

        #region InactiveColor dependency property

        internal static readonly DependencyProperty InactiveColorProperty = DependencyProperty.Register("InactiveColor", typeof (Color), typeof (StatusCustomControl), new PropertyMetadata(OnInactiveColorChanged));

        internal Color InactiveColor
        {
            get { return (Color) GetValue(InactiveColorProperty); }
            set { SetValue(InactiveColorProperty, value); }
        }

        private static void OnInactiveColorChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (StatusCustomControl) d;
            control.UpdateStatus();
        }

        #endregion

        public StatusCustomControl()
        {
            Focusable = false;
        }

        protected override void OnClick()
        {
            base.OnClick();
            if (Cell != null)
            {
                var order = Cell.Row.DataObject as Order;
                if (order != null)
                {
                    switch (order.Status)
                    {
                        case Status.Created:
                        case Status.Cancelled:
                            order.Status = Status.Active;
                            break;

                        case Status.Active:
                            order.Status = Status.Cancelled;
                            break;
                    }
                }
                var basket = Cell.Row.DataObject as Basket;
                if (basket != null)
                {
                    int activeOrders = 0;
                    foreach (Order basketOrder in basket.Orders)
                    {
                        if (Equals(Status.Active, basketOrder.Status))
                        {
                            activeOrders++;
                        }
                    }

                    //Deactivate or activate all orders
                    foreach (Order basketOrder in basket.Orders)
                    {
                        basketOrder.Status = activeOrders > 0 ? Status.Cancelled : Status.Active;
                    }
                }
            }
        }

        private void UpdateStatus()
        {
            if (Cell != null)
            {
                var order = Cell.Row.DataObject as Order;
                var basket = Cell.Row.DataObject as Basket;
                if (order != null)
                {
                    Visibility = Visibility.Visible;
                    switch (order.Status)
                    {
                        case Status.Created:
                        case Status.Cancelled:
                            Background = new SolidColorBrush(InactiveColor);
                            Content = "Start";
                            break;

                        case Status.Active:
                            Background = new SolidColorBrush(ActiveColor);
                            Content = "Stop";
                            break;
                    }
                }
                else if (basket != null)
                {
                    if (basket.Orders.Count > 0)
                    {
                        Visibility = Visibility.Visible;
                        int activeOrders = 0;
                        foreach (Order basketOrder in basket.Orders)
                        {
                            if (Equals(Status.Active, basketOrder.Status))
                            {
                                activeOrders++;
                            }
                        }

                        if (activeOrders == 0)
                        {
                            Background = new SolidColorBrush(InactiveColor);
                            Content = "Start all";
                        }
                        else if (activeOrders == basket.Orders.Count)
                        {
                            Background = new SolidColorBrush(ActiveColor);
                            Content = "Stop all";
                        }
                        else
                        {
                            double rate = ((double) activeOrders/basket.Orders.Count);

                            var stops = new GradientStopCollection();
                            stops.Add(new GradientStop(ActiveColor, 0d));
                            stops.Add(new GradientStop(ActiveColor, Math.Max(0d, rate - 0.05)));
                            stops.Add(new GradientStop(Colors.Yellow, rate));
                            stops.Add(new GradientStop(InactiveColor, Math.Min(1d, rate + 0.05)));
                            stops.Add(new GradientStop(InactiveColor, 1d));

                            Background = new LinearGradientBrush(stops, 0);
                            Content = "Stop all";
                        }
                    }
                    else
                    {
                        Visibility = Visibility.Hidden;
                        ClearValue(ContentProperty);
                        ClearValue(BackgroundProperty);
                    }
                }
                else
                {
                    Visibility = Visibility.Hidden;
                }
            }
            else
            {
                ClearValue(BackgroundProperty);
            }
        }
    }
}