﻿using System;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;
using Dapfor.Wpf.Grid.Demo.Utils;
using Dapfor.Wpf.Utils;

namespace Dapfor.Wpf.Grid.Demo.Examples.Performance.FilterPerformance
{
    /// <summary>
    /// Interaction logic for FilterPerformanceDemo.xaml
    /// </summary>
    [Example("Performance", "Filter performance", "Description_FilterPerformance")]
    public partial class FilterPerformanceDemo
    {
        #region Dependency properties

        #region VisibleRowCount

        internal static readonly DependencyProperty VisibleRowCountProperty = DependencyProperty.Register("VisibleRowCount", typeof (int), typeof (FilterPerformanceDemo));

        internal int VisibleRowCount
        {
            get { return (int) GetValue(VisibleRowCountProperty); }
            set { SetValue(VisibleRowCountProperty, value); }
        }

        #endregion

        #region FilterSpeed

        internal static readonly DependencyProperty FilterSpeedProperty = DependencyProperty.Register("FilterSpeed", typeof (string), typeof (FilterPerformanceDemo));

        internal string FilterSpeed
        {
            get { return (string) GetValue(FilterSpeedProperty); }
            set { SetValue(FilterSpeedProperty, value); }
        }

        #endregion

        #endregion

        private string _matchString;
        private int? _matchValue;
        private string _regExpression;

        public FilterPerformanceDemo()
        {
            InitializeComponent();

            grid.Filter = new Filter(IsFiltered);
        }

        public bool IsMatched(Cell cell)
        {
            bool matched = true;
            if (cell != null)
            {
                //Match values
                if (rbValueComparing.IsChecked.Value && _matchValue != null)
                {
                    matched = Equals(_matchValue.Value, cell.Value);
                }
                    //Match strings
                else if (rbStringComparing.IsChecked.Value && !string.IsNullOrEmpty(_matchString))
                {
                    matched = cell.Text.Contains(_matchString);
                }
                    //Match to regular expression
                else if (rbRegularExpression.IsChecked.Value && _regExpression != null)
                {
                    try
                    {
                        matched = Regex.IsMatch(cell.Text, "^" + _regExpression, RegexOptions.Singleline | RegexOptions.IgnoreCase);
                    }
                    catch
                    {
                        matched = true;
                    }
                }
            }
            return matched;
        }


        private bool IsFiltered(Row row)
        {
            bool filtered = false;

            //The grid calls this callback in the GUI thread each time when the value of the data object in changed
            foreach (Column column in grid.Headers[0].VisibleColumns)
            {
                //Check whether the cell matches to the filter
                filtered = !IsMatched(row[column.Id]);

                //The row can be shown - do not verify remaining columns
                if (!filtered)
                {
                    break;
                }
            }

            //The row is not filtered
            return filtered;
        }


        private void tbRowCount_TextChanged(object sender, TextChangedEventArgs e)
        {
            DateTime dt = DateTime.Now;

            //Update row count at each text changing
            tbRowCount.GetBindingExpression(TextBox.TextProperty).UpdateSource();

            TimeSpan ts = DateTime.Now - dt;
            FilterSpeed = string.Format("{0} ms", ts.TotalMilliseconds);

            UpdateVisibleRowCount();
        }


        private void UpdateVisibleRowCount()
        {
            Row firstRow = grid.Rows.TopVisibleRow;
            Row lastRow = grid.Rows.BottomVisibleRow;

            VisibleRowCount = firstRow != null && lastRow != null ? lastRow.VisibleIndex - firstRow.VisibleIndex + 1 : 0;
        }

        private void UpdateFilter()
        {
            //Get a value to match rows
            _matchValue = null;
            int v;
            if (!string.IsNullOrEmpty(tbFilter.Text) &&
                int.TryParse(tbFilter.Text.Trim(), NumberStyles.Number | NumberStyles.AllowThousands | NumberStyles.AllowLeadingWhite, CultureInfo.CurrentCulture, out v))
            {
                _matchValue = v;
            }

            //String comparing
            _matchString = tbFilter.Text;

            //Regular expression
            _regExpression = _matchString ?? string.Empty;
            _regExpression = _regExpression.Replace("*", ".*");
            _regExpression = _regExpression.Replace("?", ".");

            //Measure the time to do the filter job
            DateTime dt = DateTime.Now;

            //Notify the Grid about the filter contitions changings
            grid.FilterRefresh();

            TimeSpan ts = DateTime.Now - dt;
            FilterSpeed = string.Format("{0} ms", ts.TotalMilliseconds);
        }

        #region Private event handlers

        private void grid_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            UpdateVisibleRowCount();
        }

        private void tbFilter_TextChanged(object sender, TextChangedEventArgs e)
        {
            UpdateFilter();
        }

        private void rbValueComparing_Checked(object sender, RoutedEventArgs e)
        {
            UpdateFilter();
        }

        private void rbStringComparing_Checked(object sender, RoutedEventArgs e)
        {
            UpdateFilter();
        }

        private void rbRegularExpression_Checked(object sender, RoutedEventArgs e)
        {
            UpdateFilter();
        }

       #endregion
    }
}